﻿Imports Syncfusion.Windows.Forms.Gauge
Imports System
Imports System.Collections.Generic
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.Windows.Forms

Namespace LinearGaugeCustomization
	Public Class CustomRenderer
		Implements ILinearGaugeRenderer

		#Region "Varaibles"
		''' </summary>
		''' Gets the LinearGauge
		''' </summary>
		Private m_LinearGauge As LinearGauge
		''' <summary>
		''' Gets/Sets the Tick Distance.
		''' </summary>
		Private majorTicksDistance As Single
		''' <summary>
		''' Calculates the Minor Ticks Pixels.
		''' </summary>
		Private m_minorTicksPixels As Single
		''' <summary>
		''' Initializes the startpoint variable.
		''' </summary>
		Private startpoint As Integer

		Private majorTicksCount As Integer


		#End Region

		#Region "Properties"
		''' <summary>
		'''  Gets the LinearGauge.
		''' </summary>
		Friend ReadOnly Property LinearGauge() As LinearGauge
			Get
				Return m_LinearGauge
			End Get
		End Property
		#End Region

		#Region "Constructor"
		''' <summary>
		''' Initializes the new instance for the MetroThemeLinearGaugeRenderer class.
		''' </summary>
		''' <param name="linearGauge">LinearGauge</param>
		Public Sub New(ByVal linearGauge As LinearGauge)
			m_LinearGauge = linearGauge
			majorTicksDistance = 0
			m_minorTicksPixels = 0
			startpoint = 25
			Me.m_LinearGauge.BackgroundGradientStartColor = Color.White
			Me.m_LinearGauge.BackgroundGradientEndColor = Color.White
			m_LinearGauge.OuterFrameGradientStartColor = Color.FromArgb(17, 180, 205)
			m_LinearGauge.OuterFrameGradientEndColor = Color.FromArgb(17, 180, 205)
			m_LinearGauge.InnerFrameGradientStartColor = Color.FromArgb(180, 180, 180)
			m_LinearGauge.InnerFrameGradientEndColor = Color.FromArgb(194, 194, 194)
			Me.m_LinearGauge.NeedleColor = Color.FromArgb(17, 180, 205)
			Me.m_LinearGauge.ScaleLabelColor = Color.Gray
			majorTicksCount = 0
		End Sub
		#End Region

		#Region "ILinearGaugeMembers"

		''' <summary>f
		''' Used to draw Background Frames.
		''' </summary>
		''' <param name="Graphics">Graphics</param>
		Public Sub DrawFrame(ByVal Graphics As Graphics) Implements ILinearGaugeRenderer.DrawFrame

			Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
			Dim tmpSoundCornerRadius As Integer = Math.Min(Math.Min(25, Me.LinearGauge.Width - 2), Me.LinearGauge.Height - 2)
			Dim rect As New Rectangle(0, 0, Me.LinearGauge.Width - 1, Me.LinearGauge.Height - 1)
			Dim innerrect As New Rectangle(rect.X + 10, rect.Y + 10, Me.LinearGauge.Width - 2 * 10, Me.LinearGauge.Height - 2 * 10)

			Dim graphPath As New GraphicsPath()
			graphPath.AddArc(rect.X, rect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 180, 90)
			graphPath.AddArc(rect.X + rect.Width - tmpSoundCornerRadius, rect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 270, 90)
			graphPath.AddArc(rect.X + rect.Width - tmpSoundCornerRadius, rect.Y + rect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 0, 90)
			graphPath.AddArc(rect.X, rect.Y + rect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 90, 90)
			graphPath.AddLine(rect.X, rect.Y + rect.Height - tmpSoundCornerRadius, rect.X, rect.Y + tmpSoundCornerRadius \ 2)

			Dim graphInnerPath As New GraphicsPath()
			graphInnerPath.AddArc(innerrect.X, innerrect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 180, 90)
			graphInnerPath.AddArc(innerrect.X + innerrect.Width - tmpSoundCornerRadius, innerrect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 270, 90)
			graphInnerPath.AddArc(innerrect.X + innerrect.Width - tmpSoundCornerRadius, innerrect.Y + innerrect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 0, 90)
			graphInnerPath.AddArc(innerrect.X, innerrect.Y + innerrect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 90, 90)
			graphInnerPath.AddLine(innerrect.X, innerrect.Y + innerrect.Height - tmpSoundCornerRadius, innerrect.X, innerrect.Y + tmpSoundCornerRadius \ 2)

			Using brush1 As New LinearGradientBrush(rect, LinearGauge.OuterFrameGradientStartColor, LinearGauge.OuterFrameGradientEndColor, LinearGradientMode.Vertical)
				Graphics.FillPath(brush1, graphPath)
				Graphics.DrawPath(New Pen(LinearGauge.InnerFrameGradientEndColor, 1), graphPath)

			End Using

			Using brush2 As New LinearGradientBrush(innerrect, LinearGauge.BackgroundGradientStartColor, LinearGauge.BackgroundGradientEndColor, LinearGradientMode.Vertical)
				Graphics.FillPath(brush2, graphInnerPath)
				Graphics.DrawPath(New Pen(LinearGauge.InnerFrameGradientStartColor, 1.6F), graphInnerPath)
			End Using
		End Sub

		''' <summary>
		'''  Used to draw the Tick lines.
		''' </summary>
		''' <param name="Graphics">Graphics</param>
		Public Sub DrawLines(ByVal Graphics As Graphics) Implements ILinearGaugeRenderer.DrawLines
			Dim majorTickPen As New Pen(LinearGauge.MajorTickMarkColor, 2)
			Dim minorTickPen As New Pen(LinearGauge.MinorTickMarkColor)
			Dim brush As Brush = New SolidBrush(LinearGauge.ForeColor)
			Dim sf As New StringFormat()
			Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None

			majorTicksDistance = ((LinearGauge.MaximumValue - LinearGauge.MinimumValue) / LinearGauge.MajorDifference)
			majorTicksCount = (CInt(Math.Truncate(LinearGauge.MaximumValue - LinearGauge.MinimumValue)) \ (LinearGauge.MajorDifference)) + 1
			Dim majortickValue As Double = LinearGauge.MinimumValue
			Dim tickPosition As Single = 25F
			Dim temp1 As Single = 0
			Dim s As Single = (LinearGauge.MaximumValue - LinearGauge.MinimumValue) Mod LinearGauge.MajorDifference
			sf.Alignment = StringAlignment.Center
			sf.LineAlignment = StringAlignment.Center
			Dim minortickValue As Single = 0
			Dim tickPositionminor As Single = 0
			Dim path As New GraphicsPath()
			Dim minorcount As Integer = LinearGauge.MinorTickCount
			If LinearGauge.LinearFrameType = LinearFrameType.Vertical Then
				m_minorTicksPixels = ((Me.LinearGauge.Height - 50) / majorTicksDistance)
				Dim x As Integer = Me.LinearGauge.Width \ 2
				temp1 = 0
				For L As Integer = 1 To majorTicksCount
					Graphics.DrawLine(majorTickPen, x, Me.LinearGauge.Height - tickPosition, x - LinearGauge.MajorTicksHeight, Me.LinearGauge.Height - tickPosition)
					If LinearGauge.ShowScaleLabel Then
						Graphics.DrawString(Me.NumberToStringRepresentation(CInt(Math.Truncate(Math.Round(majortickValue, 2)))), LinearGauge.Font, brush, New PointF(x - LinearGauge.MajorTicksHeight - 25, Me.LinearGauge.Height - tickPosition), sf)
					End If
					If L = majorTicksCount Then
						minorcount = (LinearGauge.MinorTickCount * CInt(Math.Truncate(Math.Ceiling(s)))) \ LinearGauge.MajorDifference
					End If
					If majortickValue < LinearGauge.MaximumValue Then
'INSTANT VB NOTE: The variable S was renamed since Visual Basic will not allow local variables with the same name as parameters or other local variables:
						For S_Renamed As Integer = 1 To minorcount
							minortickValue = (m_minorTicksPixels / (LinearGauge.MinorTickCount + 1)) * S_Renamed
							tickPositionminor = Me.LinearGauge.Height - (minortickValue + temp1 + 25)
							Graphics.DrawLine(minorTickPen, x, CSng(tickPositionminor), x - LinearGauge.MinorTickHeight, CSng(tickPositionminor))
						Next S_Renamed
						temp1 = m_minorTicksPixels * L
					End If

					majortickValue += LinearGauge.MajorDifference
					tickPosition += m_minorTicksPixels
				Next L
				If Me.LinearGauge.MinimumValue > 0 Then
					Graphics.FillRectangle(New SolidBrush(LinearGauge.ValueIndicatorColor), Me.LinearGauge.Width \ 2 + 10, startpoint + (majorTicksDistance * m_minorTicksPixels) - ((((LinearGauge.Value - Me.LinearGauge.MinimumValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5, ((((LinearGauge.Value - Me.LinearGauge.MinimumValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2)
				Else
					Graphics.FillRectangle(New SolidBrush(LinearGauge.ValueIndicatorColor), Me.LinearGauge.Width \ 2 + 10, startpoint + (majorTicksDistance * m_minorTicksPixels) - ((((Math.Abs(Me.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5, ((((Math.Abs(Me.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2)
				End If
				brush.Dispose()
				minorTickPen.Dispose()
			Else
				m_minorTicksPixels = ((Me.LinearGauge.Width - (startpoint * 2)) / majorTicksDistance)

				Dim y As Integer = Me.LinearGauge.Height \ 2 - LinearGauge.MajorTicksHeight

				temp1 = 0
				Dim height As Integer = y + LinearGauge.MajorTicksHeight
				For L As Integer = 1 To majorTicksCount
					Graphics.DrawLine(majorTickPen, tickPosition, CSng(y) - 0.5F, tickPosition, CSng(height) - 0.5F)
					If LinearGauge.ShowScaleLabel Then
						Graphics.DrawString(Math.Round(majortickValue, 2).ToString(), LinearGauge.Font, brush, New PointF(tickPosition, y - 10), sf)
					End If
					If L = majorTicksCount Then
						minorcount = (LinearGauge.MinorTickCount * CInt(Math.Truncate(Math.Ceiling(s)))) \ LinearGauge.MajorDifference
					End If
					If majortickValue < LinearGauge.MaximumValue Then
'INSTANT VB NOTE: The variable S was renamed since Visual Basic will not allow local variables with the same name as parameters or other local variables:
						For S_Renamed As Integer = 1 To minorcount
							minortickValue = (m_minorTicksPixels / (LinearGauge.MinorTickCount + 1)) * S_Renamed
							tickPositionminor = minortickValue + temp1 + startpoint
							Graphics.DrawLine(minorTickPen, CSng(tickPositionminor), height - 1, CSng(tickPositionminor), height - LinearGauge.MinorTickHeight - 1)
						Next S_Renamed
						temp1 = m_minorTicksPixels * L
					End If

					majortickValue += LinearGauge.MajorDifference
					tickPosition += m_minorTicksPixels
				Next L
				If Me.LinearGauge.MinimumValue > 0 Then
					Graphics.FillRectangle(New SolidBrush(LinearGauge.ValueIndicatorColor), 24, Me.LinearGauge.Height \ 2 + 10, ((((Me.LinearGauge.Value - Me.LinearGauge.MinimumValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2, 5)
				Else
					Graphics.FillRectangle(New SolidBrush(LinearGauge.ValueIndicatorColor), 24, Me.LinearGauge.Height \ 2 + 10, ((((Math.Abs(Me.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2, 5)
				End If

				Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias


				brush.Dispose()
				minorTickPen.Dispose()
			End If

		End Sub

		''' <summary>
		''' Used to draw the range lines.
		''' </summary>
		''' <param name="Graphics">Graphics</param>
		Public Sub DrawRanges(ByVal Graphics As Graphics) Implements ILinearGaugeRenderer.DrawRanges

			Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
			If LinearGauge.LinearFrameType = LinearFrameType.Vertical Then
				Graphics.FillRectangle(New SolidBrush(LinearGauge.GaugeBaseColor), Me.LinearGauge.Width \ 2, startpoint - 1, 5, (((Me.majorTicksDistance)) * m_minorTicksPixels) + 2)
			Else
				Graphics.FillRectangle(New SolidBrush(LinearGauge.GaugeBaseColor), 24, Me.LinearGauge.Height \ 2, (((Me.majorTicksDistance)) * m_minorTicksPixels) + 2, 5)
			End If
			For Each ptrRange As LinearRange In Me.LinearGauge.Ranges
				Dim rvalve As Integer = CInt(Math.Truncate(Math.Ceiling(LinearGauge.MaximumValue - ptrRange.EndValue))) \ LinearGauge.MajorDifference
				If ptrRange.EndValue > ptrRange.StartValue AndAlso ptrRange.EndValue <= Me.LinearGauge.MaximumValue Then
					If Me.LinearGauge.MinimumValue >= 0 AndAlso ptrRange.StartValue < 0 Then
						Return
					End If
					Dim startValue As Single = CSng(ptrRange.StartValue)
					Dim endValue As Single = CSng(ptrRange.EndValue)
					Dim startwidth As Integer = ptrRange.StartWidth
					Dim endwidth As Integer = ptrRange.EndWidth
					If Me.LinearGauge.MinimumValue < 0 Then
						startValue = Me.LinearGauge.MinimumValue + Math.Abs(ptrRange.StartValue)
					End If
					If Me.LinearGauge.MinimumValue < 0 AndAlso ptrRange.StartValue > 0 Then
						startValue = Math.Abs(Me.LinearGauge.MinimumValue) + Math.Abs(ptrRange.StartValue)
					End If
					If Me.LinearGauge.MinimumValue < 0 AndAlso ptrRange.StartValue = 0 Then
						startValue = Math.Abs(Me.LinearGauge.MinimumValue) + Math.Abs(ptrRange.StartValue)
						startValue = (((startValue / LinearGauge.MajorDifference)) * m_minorTicksPixels)
					End If
					If LinearGauge.LinearFrameType = LinearFrameType.Horizontal Then
						Dim rect As RectangleF = RectangleF.Empty
						If ptrRange.StartValue = 0 Then
							rect = New RectangleF(startpoint + Math.Abs(startValue), Me.LinearGauge.Height \ 2 + 5, (((ptrRange.EndValue / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5)
						ElseIf ptrRange.StartValue > 0 Then
							rect = New RectangleF(25 + (((startValue / LinearGauge.MajorDifference)) * m_minorTicksPixels), Me.LinearGauge.Height \ 2 + 5, ((((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5)

						ElseIf ptrRange.StartValue < 0 Then
							startValue = Me.LinearGauge.MinimumValue - ptrRange.StartValue
							rect = New RectangleF(25 + (((Math.Abs(startValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), Me.LinearGauge.Height \ 2, ((((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5)
						End If
						Dim path As New GraphicsPath()
						Dim pt1, pt2, pt3, pt4 As PointF
						pt1 = New PointF(rect.X, rect.Y)
						pt2 = New PointF(rect.X, rect.Y - startwidth)
						pt4 = New PointF(rect.Right, rect.Y)
						pt3 = New PointF(rect.Right, rect.Y - endwidth)
						path.AddPolygon(New PointF() { pt1, pt2, pt3, pt4 })
						path.Reverse()
						Graphics.FillPath(New SolidBrush(ptrRange.Color), path)
					Else
						Dim rangeheight As Single = (ptrRange.EndValue / LinearGauge.MajorDifference) * m_minorTicksPixels
						Dim endvalueRangeHeight As Single = 0F
						If Me.LinearGauge.MinimumValue < 0 Then
							rangeheight = ((Math.Abs(Me.LinearGauge.MinimumValue) + ptrRange.EndValue) / LinearGauge.MajorDifference) * m_minorTicksPixels
						End If
						endvalueRangeHeight = rangeheight
						If Me.LinearGauge.MinimumValue < 0 AndAlso ptrRange.StartValue = 0 Then
							endvalueRangeHeight = (((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference) * m_minorTicksPixels)
						End If
						Dim rect As RectangleF = RectangleF.Empty
						If ptrRange.StartValue = 0 Then
							rect = New RectangleF(Me.LinearGauge.Width \ 2 + 5, startpoint + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 5, endvalueRangeHeight)
						ElseIf ptrRange.StartValue > 0 Then
							rect = New RectangleF(Me.LinearGauge.Width \ 2 + 5, startpoint + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 5, (((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference) * m_minorTicksPixels))
						ElseIf ptrRange.StartValue < 0 Then
							rect = New RectangleF(Me.LinearGauge.Width \ 2 + 5, startpoint + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 5, (((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference) * m_minorTicksPixels))
						End If
						Dim path As New GraphicsPath()
						Dim pt1, pt2, pt3, pt4 As PointF
						pt1 = New PointF(rect.X, rect.Y)
						pt2 = New PointF(rect.X - endwidth, rect.Y)
						pt4 = New PointF(rect.X, rect.Bottom)
						pt3 = New PointF(rect.X - startwidth, rect.Bottom)
						path.AddPolygon(New PointF() { pt1, pt2, pt3, pt4 })
						path.Reverse()
						Graphics.FillPath(New SolidBrush(ptrRange.Color), path)
					End If
				End If
			Next ptrRange
		End Sub
		''' <summary>
		''' Used to draw the Pointers. 
		''' </summary>
		''' <param name="Graphics">Graphics</param>
		Public Sub DrawPointer(ByVal Graphics As Graphics) Implements ILinearGaugeRenderer.DrawPointer
			Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality
			Dim path As New GraphicsPath()
			Dim a As Integer = 0
			If Me.LinearGauge.MinimumValue < 0 Then
				a = CInt(Math.Truncate(Math.Ceiling((((Math.Abs(Me.LinearGauge.MinimumValue) + LinearGauge.Value) / CSng(LinearGauge.MajorDifference)) * m_minorTicksPixels))))
			Else
				a = CInt(Math.Truncate(Math.Ceiling((Math.Abs(LinearGauge.Value - Me.LinearGauge.MinimumValue) / CSng(LinearGauge.MajorDifference)) * m_minorTicksPixels)))
			End If
			Dim y As Integer = (Me.LinearGauge.Height \ 2 + LinearGauge.MajorTicksHeight) - LinearGauge.MajorTicksHeight
			Dim pt1, pt2, pt3 As Point
			If LinearGauge.LinearFrameType = LinearFrameType.Vertical Then
				a = 10 + CInt(Math.Truncate(Math.Ceiling((majorTicksDistance * m_minorTicksPixels)))) - a

				If LinearGauge.PointerPlacement = Placement.Near Then
					pt1 = New Point(Me.LinearGauge.Width \ 2 - 15, 10 + a)
					pt2 = New Point(Me.LinearGauge.Width \ 2 - 15, 20 + a)
					pt3 = New Point(Me.LinearGauge.Width \ 2, 15 + a)
				ElseIf LinearGauge.PointerPlacement = Placement.Center Then
					pt1 = New Point(Me.LinearGauge.Width \ 2 + 20, 10 + a)
					pt2 = New Point(Me.LinearGauge.Width \ 2 + 20, 20 + a)
					pt3 = New Point(Me.LinearGauge.Width \ 2 + 5, 15 + a)
				Else
					pt1 = New Point(Me.LinearGauge.Width \ 2 + 28, 10 + a)
					pt2 = New Point(Me.LinearGauge.Width \ 2 + 28, 20 + a)
					pt3 = New Point(Me.LinearGauge.Width \ 2 + 13, 15 + a)
				End If

			Else
				a = a + 10

				If LinearGauge.PointerPlacement = Placement.Near Then
					pt1 = New Point(10 + a, y - 15)
					pt2 = New Point(20 + a, y - 15)
					pt3 = New Point(15 + a, y)
				ElseIf LinearGauge.PointerPlacement = Placement.Center Then
					pt1 = New Point(10 + a, y + 20)
					pt2 = New Point(20 + a, y + 20)
					pt3 = New Point(15 + a, y + 5)
				Else
					pt1 = New Point(10 + a, y + 27)
					pt2 = New Point(20 + a, y + 27)
					pt3 = New Point(15 + a, y + 12)
				End If

			End If
			path.AddPolygon(New Point() { pt1, pt2, pt3 })
			Graphics.FillPath((New SolidBrush(LinearGauge.NeedleColor)), path)
		End Sub

		''' <summary>
		''' Updates the Render
		''' </summary>
		''' <param name="PaintEventArgs">PaintEventArgs</param>
		Public Sub UpdateRenderer(ByVal PaintEventArgs As PaintEventArgs) Implements ILinearGaugeRenderer.UpdateRenderer
			DrawFrame(PaintEventArgs.Graphics)
			DrawLines(PaintEventArgs.Graphics)
			DrawRanges(PaintEventArgs.Graphics)
			DrawPointer(PaintEventArgs.Graphics)
		End Sub

		''' <summary>
		''' To convert number as String
		''' </summary>
		''' <param name="n"></param>
		''' <returns></returns>
		Public Function NumberToStringRepresentation(ByVal n As Integer) As String
			If n < 0 Then
				Return "Minus " & NumberToStringRepresentation(-n)
			ElseIf n = 0 Then
				Return ""
			ElseIf n <= 19 Then
				Return New String() {"One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", "Nine", "Ten", "Eleven", "Twelve", "Thirteen", "Fourteen", "Fifteen", "Sixteen", "Seventeen", "Eighteen", "Nineteen"}(n - 1) & " "
			ElseIf n <= 99 Then
				Return New String() {"Twenty", "Thirty", "Forty", "Fifty", "Sixty", "Seventy", "Eighty", "Ninety"}(n \ 10 - 2) & " " & NumberToStringRepresentation(n Mod 10)
			ElseIf n <= 199 Then
				Return "One Hundred " & NumberToStringRepresentation(n Mod 100)
			ElseIf n <= 999 Then
				Return NumberToStringRepresentation(n \ 100) & "Hundreds " & NumberToStringRepresentation(n Mod 100)
			ElseIf n <= 1999 Then
				Return "One Thousand " & NumberToStringRepresentation(n Mod 1000)
			ElseIf n <= 999999 Then
				Return NumberToStringRepresentation(n \ 1000) & "Thousands " & NumberToStringRepresentation(n Mod 1000)
			ElseIf n <= 1999999 Then
				Return "One Million " & NumberToStringRepresentation(n Mod 1000000)
			ElseIf n <= 999999999 Then
				Return NumberToStringRepresentation(n \ 1000000) & "Millions " & NumberToStringRepresentation(n Mod 1000000)
			ElseIf n <= 1999999999 Then
				Return "One Billion " & NumberToStringRepresentation(n Mod 1000000000)
			Else
				Return NumberToStringRepresentation(n \ 1000000000) & "Billions " & NumberToStringRepresentation(n Mod 1000000000)
			End If
		End Function

		#End Region
	End Class
End Namespace
